%PHASEC_MINIMIZER  This script minimizes the parameters in the
%  general solution for Section I and Section III of Phase C
%  simultaneously

close all;
clear all;

% uncomment for the fish data 
%rawdata = load('b1.mat');

% uncomment for the clean fish data:
load('b1_clean');
rawdata(:, 1) = [1:1024]';
rawdata(:, 2) = u1';

% the time boundaries
T10 = 780+1;  % these are off by one!
T1F = 790+1;
T20 = 806+1;
T2F = 879+1;

time1offset = rawdata(T10,1);
time1 = rawdata(T10:T1F, 1) - time1offset;
depth1 = rawdata(T10:T1F, 2);

time2offset = rawdata(T20,1);
time2 = rawdata(T20:T2F, 1) - time2offset;
depth2 = rawdata(T20:T2F, 2);

% this is just used for plotting
alldepth = rawdata(T10:T2F, 2);
alltime = rawdata(T10:T2F, 1);

% number of times to restart optimization.  basically we want to
% avoid local minima
N = 100;

% stores parameters values and the associated cost value
Xs = zeros(N,6);
fvals = zeros(N,1);

figure(1);
clf;
plot(alltime, alldepth, 'b-');
hold on;

for n = 1:N
  % these are initial values for the parameters
  X0 = [.04*rand+.02 .4*rand .1*rand+.9 -1*rand -.4*rand pi*rand-pi/2];
 
  [X,fval] = fminsearch(@pc_sec1_cost_5, X0, [], time1, depth1, time2, depth2);
  Xs(n,:) = X;
  fvals(n) = fval;  
  
  delta = X(1);
  omega = X(2);
  D = X(3);
  A = X(4);
  B = X(5);
  phi2 = X(6);
  
  phi = atan(-delta/omega);

  fval
  y1 = A*exp(-delta*time1) .* cos(omega*time1 + phi) + D;
  y2 = B*exp(-delta*time2) .* cos(omega*time2 + phi2) + D;
  plot(time1+time1offset, y1, 'r-');
  plot(time2+time2offset, y2, 'm-');
  pause(0);
  if(mod(n, 10) == 0)
      disp(sprintf('%10d/%d done', n, N));
  end
end

% find the index of one of the global minima
[minfval, js] = min(fvals);
j = js(1);

minfval
delta = Xs(j,1)
omega = Xs(j,2)
D = Xs(j,3)
A = Xs(j,4)
B = Xs(j,5)
phi2 = Xs(j,6)
phi = atan(-delta/omega);

y1 = A*exp(-delta*time1) .* cos(omega*time1+phi) + D;
y2 = B*exp(-delta*time2) .* cos(omega*time2+phi2) + D;

figure(2);
clf;
plot(alltime, alldepth, 'b-');
hold on;
plot(time1+time1offset, y1, 'r-');
plot(time2+time2offset, y2, 'r-');
pause(0);

figure(3);
clf;
plot(sort(fvals));

save pc_plot HACK_y1 HACK_y1p HACK_y2 HACK_y2p T10 T1F T20 T2F delta ...
    omega D A B phi phi2 minfval time1 time2 time1offset time2offset ...
    alltime alldepth
